/*
* Copyright (c) 2022 TECNALIA RESEARCH & INNOVATION.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of Affero General Public License (AGPL) version 3
* which accompanies this distribution, and is available at

* https://www.gnu.org/licenses/agpl-3.0.en.html
*
* Contributors:
*
* Gonzalo Lazaro, Jose Manuel Rio, Alejandro Rodriguez, Sonia Bilbao from Tecnalia
*
* Initially developed in the context of URBANITE EU project
* www.urbanite-project.eu
*/
package com.tecnalia.urbanite.anonymize.rest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.tecnalia.urbanite.anonymize.model.City;
import com.tecnalia.urbanite.anonymize.model.Location;
import com.tecnalia.urbanite.anonymize.service.AnonymizerService;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@Tag(name = "Anonymize", description = "Operations to Anonymize Data")
public class AnonymizerAPI {

	@Autowired
	AnonymizerService anonymizerService;

    @RequestMapping(method = RequestMethod.POST, value = "/anonymize/{city}/{model}/{property}/{value}")
    @ApiResponses(value = { 
    		@ApiResponse(responseCode = "200", description = "Successful operation."),
    		@ApiResponse(responseCode = "400", description = "Bad request. (User unknow)")})  
    @Operation(summary = "Anonymize property that has certain value", description = "Change the value {value} of the {property} for a hash..\\n\""
    		 + "<br/><br/><b>THIS SERVICE HAS BEEN DISABLED IN THE TEST ENVIRONMENT</b> ")
    public ResponseEntity<String> anonymizeUser(
    		@Parameter(description = "City where is the data to be processed") @PathVariable City city, 
			 @Parameter(description = "Model that is going to be anonymized.", example = "vehicle") @PathVariable String model,
    		 @Parameter(description = "Property of the model that is going to be anonymized.", example = "name") @PathVariable String property, 
    		 @Parameter(description = "Only the properties with this value are going to be modified.", example = "Jonh") @PathVariable String value) throws Exception {
       
    	return anonymizerService.anonymizeProperty(city.name(),model,property,value);
            
    }    
    
    @RequestMapping(method = RequestMethod.POST, value = "/removePointsOutOfArea/{city}/{model}/{property}/{value}")
    @ApiResponses(value = { 
    		@ApiResponse(responseCode = "200", description = "Successful operation."),
    		@ApiResponse(responseCode = "400", description = "Bad request. ")})  
    @Operation(summary = "Given an input polygon, delete the points outside that polygon ", description = "Given an input polygon,{location} delete the points outside that polygon. The points are those defined by the city, property and value model.\n"
    		+ "And they must have a \"location\" field with an array of \"coordinates\"."
    		+ "<br/><br/><b>THIS SERVICE HAS BEEN DISABLED IN THE TEST ENVIRONMENT</b> ")
    public ResponseEntity<String> removePointsOutOfArea(
    		 @Parameter(description = "Points array to be validated ", example = "{\"location\": {\n"
    		 		+ "			\"type\": \"LineString\",\n"
    		 		+ "			\"coordinates\": [\n"
    		 		+ "				[15.5612754821777, 38.2057461528513],\n"				
    		 		+ "				[15.4648876190186, 38.1668198052653],\n"
    		 		+ "				[15.4643297195435, 38.1587890805601]\n"
    		 		+ "			]\n"
    		 		+ "		}}") @RequestBody  Location location,
    		 @Parameter(description = "City where is the data to be processed") @PathVariable City city, 
			 @Parameter(description = "Model that is going to be anonymized.", example = "vehicle") @PathVariable String model,
    		 @Parameter(description = "Property of the model that is going to be anonymized.", example = "name") @PathVariable String property, 
    		 @Parameter(description = "Only the properties with this value are going to be modified.", example = "Jonh") @PathVariable String value) throws Exception {
       
    	return anonymizerService.removePointsOutOfArea(location,city.name(),model,property,value);
            
    }    	
    
    
    @RequestMapping(method = RequestMethod.POST, value = "/generateAnonymousVehicleTripFromVehiclePoints/{city}")
    @ApiResponses(value = { 
    		@ApiResponse(responseCode = "200", description = "Successful operation."),
    		@ApiResponse(responseCode = "400", description = "Bad request. ")})    
    @Operation(summary = "Generate anonymiced routes from vehicles points.", description = "The service get the messina Vehicle points of ebikes and generate GPS Routes, then anonymice this routas deleting the "
    		+ " {meters} of the beginnig and the end.")
    public ResponseEntity<String> generateAnonymousVehicleTripFromVehiclePoints(   
    		 					@Parameter(description = "City where is the data to be processed", example = "messina") @PathVariable City city, 
						   		@Parameter(description = "Meters to delete", example = "50", required = false) @RequestParam(required = false)  double meters
								) throws Exception {
    		
    	return anonymizerService.generateAnonymousVehicleTripFromVehiclePointsString(city.name(),"vehicle","name", meters);
            
    }    	
}
