package com.tecnalia.urbanite.anonymize.service;

import java.awt.geom.Path2D;
import java.io.IOException;
import java.net.URISyntaxException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.tecnalia.urbanite.anonymize.model.Location;
import com.tecnalia.urbanite.anonymize.utils.Configuration;
import com.tecnalia.urbanite.anonymize.utils.Crypto;
import com.tecnalia.urbanite.anonymize.utils.DataUtils;

import sml.gps.datamodel.GPSPoint;
import sml.gps.datamodel.GPSRoute;
import sml.utils.GeoDistance;
import sml.visualizer.gui.functions.DivideInTraysCover;

@Service
public class AnonymizerService {
	
	@Autowired
	DataService dataService;
	
	
	public ResponseEntity<String> anonymizeProperty (String city,String model, String property, String valueToAnonimize) {
		
		try {
			JSONArray objects  = dataService.getObjectsByName(property,valueToAnonimize, model, city);
			if (objects == null || objects.length() ==0) {
				return new ResponseEntity<>("User not found !!", HttpStatus.BAD_REQUEST);
			}
			else {
				
				for (int i=0; i < objects.length();i++) {
					JSONObject object = objects.getJSONObject(i);
					String propertyAnonimizied = Crypto.generateHash((String) object.get(property));
					object.put(property, propertyAnonimizied);
					//JSONObject result = dataService.updateObjects(object, model, city,object.getString("id"));
					
					/*if (result.has("Error")) {
						return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !!\n " + result.getString("Error"), HttpStatus.INTERNAL_SERVER_ERROR);
					}
					else {*/
						System.out.println("THIS SERVICE HAS BEEN DISABLED IN THE TEST ENVIRONMENT");//result.toString());
					/*}*/
					
				}
			}
			return new ResponseEntity<>("User anonymizied successfully !!\n", HttpStatus.OK);
			
		} catch (JSONException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (URISyntaxException e) {
			
			e.printStackTrace();
			return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (IOException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (InterruptedException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (NoSuchAlgorithmException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		}
		
	}
	
	public ResponseEntity<String> generateAnonymousVehicleTripFromVehiclePointsString (String city,String model, String property, double distanceInMeters)  {
		try {
			JSONObject object  = dataService.getDistinctObject(model, city, property);//messina,vehicle,name
			JSONArray objects = (JSONArray) object.get("values");
			String respuesta ="";
			if (objects.length() > 0) {
				for (int i=0;i<objects.length();i++) {
					
					String vehicleName = objects.getString(i);
					//if (!vehicleName.equals("352503093621149")) continue;
					//respuesta= respuesta+vehicleName+"-->";
					
					JSONArray vehiclePoints  = dataService.getObjectsByName(property,vehicleName, model, city);			
					vehicleName = Crypto.generateHash(vehicleName);
					//respuesta= respuesta+vehicleName+"\n";
					GPSRoute route = new GPSRoute();
					ArrayList<GPSPoint> puntos = new ArrayList<GPSPoint>();
					route.setName(vehicleName);
					route.setNodes(puntos);
					for (int ipoint = 0 ; ipoint< vehiclePoints.length();ipoint++) {
						boolean isvalid = DataUtils.isValidVehiclePoint(vehiclePoints.getJSONObject(ipoint));
						if (isvalid) {
							puntos.add(DataUtils.getGpsPointFromVehiclePoint(vehiclePoints.getJSONObject(ipoint)));
						}
					}
					if (route == null || route.getNodes().isEmpty()) {
						continue;
					}
					DivideInTraysCover ditc = new DivideInTraysCover(route);
					if (ditc == null || ditc.doFindingTrays().isEmpty()) {
						continue;
					}
					ArrayList<GPSRoute> rutasAntesdeAnonimizar = ditc.doFindingTrays();
					int borrados = 0;
					//int b=0;
					JSONArray arrObs = new JSONArray();
					for (GPSRoute rutaaAnonymizar:rutasAntesdeAnonimizar) {
						
						ArrayList<GPSPoint> nodos = rutaaAnonymizar.getArrayNodes();
						if (nodos.size() < 3) {
							System.out.println("Ruta with few points. removed");
							borrados++;
							continue;
						}
						double distanciaruta = GeoDistance.CalcDistance(nodos.get(0), nodos.get(nodos.size()-1))*1000;
						if (distanciaruta < distanceInMeters*2) {
							System.out.println("Ruta too short "+distanciaruta+". removed");
							borrados++;
							continue;
						}
						else {
														
							//DataUtils.printGPSRoute(rutaaAnonymizar);
							ArrayList<GPSPoint> puntosAnonymizadosBegin = DataUtils.removeDistanceFromBegin(nodos, distanceInMeters);
							ArrayList<GPSPoint> puntosAnonymizados = DataUtils.removeDistanceFromEnd(puntosAnonymizadosBegin, distanceInMeters);

							if (puntosAnonymizados.size() < 3) {
								System.out.println("Ruta borrada al anonymizar");
								borrados++;
								continue;
							}
							//b++;
							GPSRoute rutaAnonymizada = new GPSRoute();
							rutaAnonymizada.setName(vehicleName);
							rutaAnonymizada.setNodes(puntosAnonymizados);
							arrObs.put(DataUtils.gpsRouteToNGSINode(rutaAnonymizada).toString());
							//DataUtils.printGPSRoute(rutaAnonymizada);
							//if (b==2 )break;
						}
						
					}
					respuesta =DataUtils.storeDataFlow("gtfsShape",arrObs.toString(),city,Configuration.getExporterUrl());
					
					System.out.println("Total rutas bici:"+vehicleName+"="+rutasAntesdeAnonimizar.size()+" Eliminadas:"+borrados);
					
				}
				return new ResponseEntity<>(respuesta, HttpStatus.OK);
			}
			else {
				return new ResponseEntity<>("Error generating anonymous routes, no data available !!", HttpStatus.BAD_REQUEST);	
			}
			
					
		} catch (JSONException e) {
			e.printStackTrace();
			return new ResponseEntity<>("Error while generateAnonymousVehicleTripFromVehiclePointsString:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (URISyntaxException e) {
			e.printStackTrace();
			return new ResponseEntity<>("Error while generateAnonymousVehicleTripFromVehiclePointsString:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (IOException e) {
			e.printStackTrace();
			return new ResponseEntity<>("Error while generateAnonymousVehicleTripFromVehiclePointsString:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (InterruptedException e) {
			e.printStackTrace();
			return new ResponseEntity<>("Error while generateAnonymousVehicleTripFromVehiclePointsString:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return new ResponseEntity<>("Error while generateAnonymousVehicleTripFromVehiclePointsString:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	public ResponseEntity<String> removePointsOutOfArea(Location location, String city,String model, String property,String value) {
		try {
			JSONArray objects  = dataService.getObjectsByName(property,value, model, city);
			List<List<Double>> coordinateslist = location.getCoordinates();
			if (!coordinateslist.isEmpty()) {
				
				Path2D path = new Path2D.Double();
				for (int i=0; i < coordinateslist.size();i++) {
					List<Double> coordinates = coordinateslist.get(i);
					if (i==0) {
						path.moveTo(coordinates.get(0), coordinates.get(1));
					}
					else {
						path.lineTo(coordinates.get(0), coordinates.get(1));
					}					
				}
				path.closePath();
				
				
				for (int i=0; i < objects.length();i++) {
					JSONObject object = objects.getJSONObject(i);
					JSONObject josnlocation =object.getJSONObject("location");
					JSONArray coordinate = josnlocation.getJSONArray("coordinates");
					if (!path.contains(coordinate.getDouble(0), coordinate.getDouble(1))) {
						
						System.out.println("Not Contains!");
						/*JSONObject result =dataService.deleteObject(model, city,object.getString("id"));						
						if (result.has("Error")) {
							return new ResponseEntity<>("Error while anonymizig:" + property + ". Please try again !!\n " + result.getString("Error"), HttpStatus.INTERNAL_SERVER_ERROR);
						}
						else {
							System.out.println(result.toString());
						}*/
					}
					else{
						System.out.println("Contains!");
					}
					
					
				}
				
				return new ResponseEntity<>("Points removed successfully !!", HttpStatus.OK);		
			
				
			}
			
			
			else {
				return new ResponseEntity<>("Coordinates not defined.", HttpStatus.BAD_REQUEST);
			}
			
			
		} catch (JSONException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while removePointsOutOfArea:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (URISyntaxException e) {
			
			e.printStackTrace();
			return new ResponseEntity<>("Error while removePointsOutOfArea:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (IOException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while removePointsOutOfArea:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (InterruptedException e) {			
			e.printStackTrace();
			return new ResponseEntity<>("Error while removePointsOutOfArea:" + property + ". Please try again !! " + e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		} 

		
	}
}
