package com.tecnalia.urbanite.anonymize.utils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.OffsetDateTime;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;

import sml.gps.datamodel.GPSPoint;
import sml.gps.datamodel.GPSRoute;
import sml.utils.GeoDistance;

public class DataUtils {
	public static boolean isValidVehiclePoint(JSONObject vehicle) throws JSONException {
		String observationDateTime = vehicle.getString("observationDateTime");
        ZonedDateTime dateTime = getZonedDateTimeFromString(observationDateTime);
        
       if (dateTime ==null) {
            System.out.println ("Descatartado:"+observationDateTime);
            return false;            
        }		
        if (dateTime.getYear() < 2020 || dateTime.getYear() > 2021){
            System.out.println ("Descatartado:"+dateTime.getYear());
            return false; 
        }
        else if (dateTime.getMonthValue() < 8 && dateTime.getYear() == 2020  ) {
            System.out.println ("Descatartado:"+dateTime.getYear()+"-"+dateTime.getMonthValue());
            return false; 
            
        }
        else{
        	return true;
        }        
		
	}
	public static GPSPoint getGpsPointFromVehiclePoint(JSONObject vehicle) throws JSONException {
		
		String observationDateTime = vehicle.getString("observationDateTime");
		ZonedDateTime dateTime = getZonedDateTimeFromString(observationDateTime);
		if (dateTime == null) {
        	System.out.println ("Descatartado:"+observationDateTime);
        	return null;            
        }			
        Long time =    dateTime.toEpochSecond() *1000;         
    	double lon  = vehicle.getJSONObject("location").getJSONArray("coordinates").getDouble(1);
        double lat =vehicle.getJSONObject("location").getJSONArray("coordinates").getDouble(0);
        double speed = vehicle.getDouble("speed");
        GPSPoint p =new GPSPoint(time,lat,lon,speed);
        p.setTime(dateTime.toEpochSecond());
        return p;
 		
	}
	
	private static ZonedDateTime getZonedDateTimeFromString (String zonedDateTime) {
		ZonedDateTime dateTime = null;
        try {                  
            dateTime =OffsetDateTime.parse(zonedDateTime).toZonedDateTime();
            return dateTime;
        }
	    catch (java.time.format.DateTimeParseException e){
	    	System.out.println ("Descatartado:"+zonedDateTime+ " -- "+e.getMessage());
	        return null;            
	    }		
		
	}
	public static void printGPSRoute(GPSRoute route) {
		String respuesta = "";
		ArrayList<GPSPoint> nodos = route.getArrayNodes();
		double distanciarutaanonimizada = GeoDistance.CalcDistance(nodos.get(0), nodos.get(nodos.size()-1))*1000;
		System.out.println("Ruta con distancia: "+distanciarutaanonimizada+" Nuemro de puntos:"+nodos.size()+".");
		
		for(int j=0;j<nodos.size();j++) {
			GPSPoint nodo = nodos.get(j);								
			respuesta = respuesta + nodo.getTime()+","+nodo.getLatitude()+","+nodo.getLongitude()+","+nodo.getSpeed()+"\n";
			
		}
		respuesta = respuesta +"-------\n";
		System.out.println (respuesta);

		
	}
	public static ArrayList<GPSPoint> removeDistanceFromBegin(ArrayList<GPSPoint> nodos,double distanceInMeters){
		ArrayList<GPSPoint> puntosAnonymizados = new ArrayList<GPSPoint>();
		double distanciaBorrada=0;
		for(int j=0;j<nodos.size();j++) {
			//if (j==0) continue;
			if (distanciaBorrada >= distanceInMeters) {
				puntosAnonymizados.add(nodos.get(j));
			}							
			else if  (j != nodos.size()-1) {
				double parcial = GeoDistance.CalcDistance(nodos.get(j), nodos.get(j+1))*1000;
				System.out.println("distanciaBorrada:"+parcial);
				distanciaBorrada = distanciaBorrada + parcial;
			}
		}
		System.out.println("Total distanciaBorrada:"+distanciaBorrada);
		return puntosAnonymizados;
	}
	public static ArrayList<GPSPoint> removeDistanceFromEnd(ArrayList<GPSPoint> nodos,double distanceInMeters){
		ArrayList<GPSPoint> puntosAnonymizados = new ArrayList<GPSPoint>();
		double distanciaBorrada=0;
		for(int j=nodos.size()-1 ;j>=0;j--) {
			//if (j==0) continue;
			if (distanciaBorrada >= distanceInMeters) {
				puntosAnonymizados.add(nodos.get(j));
			}							
			else if  (j > 0) {
				double parcial = GeoDistance.CalcDistance(nodos.get(j-1), nodos.get(j))*1000;
				System.out.println("distanciaBorrada:"+parcial);
				distanciaBorrada = distanciaBorrada + parcial;
			}
		}
		Collections.reverse(puntosAnonymizados);
		System.out.println("Total distanciaBorrada:"+distanciaBorrada);
		return puntosAnonymizados;
	}
	public static ObjectNode gpsRouteToNGSINode(GPSRoute route) {
		ArrayList<GPSPoint> nodos = route.getArrayNodes();
		Long time = nodos.get(0).getTime();
		Date dd = new Date();
		String id  = ""+dd.getTime();
		
		if (time != null) {
			id  = "" + time;
			dd = new Date(time*1000);
		}
		
		DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");		
        String dateObserved = df.format(dd);
		String name = route.getName();
		ObjectNode node = new ObjectMapper().createObjectNode()
                .put("id", "urn:ld:GtfsShape:messina:ebiketrajectories:" + name+":"+id)
                .put("type", "GtfsShape")
                .put("alternateName", "ebiketrajectories")
                .put("dateObserved", dateObserved)
                .put("name", name);
                
        ArrayNode arrayNodeContext = new ObjectMapper().createArrayNode()
                .add("https://smartdatamodels.org/context.jsonld")
                .add("https://uri.etsi.org/ngsi-ld/v1/ngsi-ld-core-context.jsonld");

        node.set("@context", arrayNodeContext);
        ObjectNode location = new ObjectMapper().createObjectNode();
        
        //ArrayNode coordinates = new ObjectMapper().createArrayNode();
        ArrayNode poligons = new ObjectMapper().createArrayNode();
        //ArrayNode trozos = new ObjectMapper().createArrayNode();
        for (GPSPoint nodo:nodos) {
        	ArrayNode point = new ObjectMapper().createArrayNode();
        	point.add(nodo.getLatitude());
        	point.add(nodo.getLongitude());
        	poligons.add(point);
        }
        //trozos.add(poligons);
        //coordinates.add(poligons);
        location.set("coordinates",poligons);
        location.put("type","LineString");
        node.set("location",location);
		
		return node;
		
		
	}
	   public static String storeDataFlow(String model, String jsonInput, String piloto,String hurl) {
	       HttpURLConnection conn = null;
	        try {
	            // Install the all-trusting trust manager
	            SSLContext sc = SSLContext.getInstance("SSL");
	            sc.init(null, trustAllCerts, new java.security.SecureRandom());
	            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());

	            // Install the all-trusting host verifier
	            HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);
	//http://localhost/data/insertTData/touristTrip/bilbao
	//https://helsinki.urbanite.esilab.org/data/insertTData/TransportStation/helsinki
	//https://amsterdam.urbanite.esilab.org/data/insertTData/pointOfInterest/amsterdam
	            URL url = new URL(hurl + "/data/insertTData/"+model+"/"+piloto);
	            conn = (HttpURLConnection) url.openConnection();
	            conn.setDoOutput(true);
	            conn.setRequestMethod("POST");
	            conn.setRequestProperty("Content-Type", "application/json");

	            OutputStream os = conn.getOutputStream();
	            os.write(jsonInput.getBytes());
	            os.flush();
	          
	            BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

	            String output;
	            
	            while ((output = br.readLine()) != null) {

	                if (!output.contains("\"notInserted\":[]")) {
	                    JSONObject res = new JSONObject(output);
	                    System.out.println(res.getJSONArray("notInserted").toString());
	                    System.out.println("Error in storeDataFlow model= " + model );                    
	                    return output;
	                }
	                else {
	                    System.out.println("Correct!");
	                }

	            }

	            
	            return output;

	        } catch (Exception e) {
	            System.err.println("Error in storeDataFlow model= " + model+" "+e.getMessage());// + " \njsonInput " + jsonInput);
	            System.err.println(e.getMessage());
	            //System.exit(0);
	            return "Error in storeDataFlow model= " + model+" "+e.getMessage();

	        }
	        finally {
	            if (conn!=null)conn.disconnect();
	        }

	    }     
    private static TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
        public java.security.cert.X509Certificate[] getAcceptedIssuers() {
            return null;
        }

        public void checkClientTrusted(X509Certificate[] certs, String authType) {
        }

        public void checkServerTrusted(X509Certificate[] certs, String authType) {
        }
    }
    };

    // Create all-trusting host name verifier
    private static HostnameVerifier allHostsValid = new HostnameVerifier() {
        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };
}
