package sml.geoUtils;

import java.util.ArrayList;
import sml.geoUtils.geometry.BBox;
import sml.geoUtils.geometry.IPointGeo;
import sml.geoUtils.geometry.IPolyline;
import sml.geoUtils.geometry.PointGeo;
import sml.geoUtils.geometry.Segment;

public class Topology {



     public IPointGeo projectPointSphere(IPointGeo toProject, Segment<IPointGeo> projectTo){
         Vector3 a = new Vector3(projectTo.getStartPoint());
         Vector3 b = new Vector3(projectTo.getEndPoint());
         Vector3 c = new Vector3(toProject);
         
         Vector3 greatCircleN  = Vector3.crossProduct(a,b);
         Vector3 greatCircleCN = Vector3.crossProduct(c,greatCircleN);
         Vector3 projected     = Vector3.crossProduct(greatCircleN, greatCircleCN);
         projected.normalize();
         
         PointGeo result = projected.toSpherical();
         
         Double apDistance = Calculations.getDistance2D(projectTo.getStartPoint(), result);
         Double bpDistance = Calculations.getDistance2D(projectTo.getEndPoint(), result);
         if ( apDistance + bpDistance - Calculations.getLength((IPolyline<IPointGeo>)projectTo) < 0.01 ){ return result; }
         else{
            if(Calculations.getDistance2D(projectTo.getStartPoint(),toProject) < Calculations.getDistance2D(projectTo.getEndPoint(),toProject)){
                 return projectTo.getStartPoint();
            }else{
                 return projectTo.getEndPoint();
            }
         }
     
     }

     public static IPointGeo projectPoint  (IPointGeo toProject, Segment<IPointGeo> projectTo){
	    Double u = ((projectTo.getEndPoint().getLongitude() - projectTo.getStartPoint().getLongitude()) * 
                        (toProject.getLongitude()               - projectTo.getStartPoint().getLongitude()) 
		      + (projectTo.getEndPoint().getLatitude()  - projectTo.getStartPoint().getLatitude()) * 
                        (toProject.getLatitude()                - projectTo.getStartPoint().getLatitude())
                       ) /
		        (  Math.pow(projectTo.getEndPoint().getLongitude() - projectTo.getStartPoint().getLongitude(), 2) 
                         + Math.pow(projectTo.getEndPoint().getLatitude()  - projectTo.getStartPoint().getLatitude(),  2)
                        );
            
            if ( u <= 0.0 ) return projectTo.getStartPoint();
            if ( u >= 1.0 ) return projectTo.getEndPoint();
            Double lon = projectTo.getStartPoint().getLongitude() + u *(projectTo.getEndPoint().getLongitude() - projectTo.getStartPoint().getLongitude());
            Double lat = projectTo.getStartPoint().getLatitude()  + u *(projectTo.getEndPoint().getLatitude()  - projectTo.getStartPoint().getLatitude());
            return new PointGeo(lat,lon);
     }
     public static IPointGeo projectPoint  (IPointGeo point, IPolyline<IPointGeo> line){
          Double minDistance = Double.POSITIVE_INFINITY;
          IPointGeo closestPoint = null;
          for(Segment segment : line.getSegments()){
              IPointGeo projected = projectPoint(point,segment);
              Double distance = Calculations.getDistance2D(point, projected);
              if( distance < minDistance ){
                minDistance = distance;
                closestPoint = projected;
              }
          }
          return closestPoint;
     }
     public static IPointGeo projectPoint  (IPointGeo point, IPolyline<IPointGeo> line, Segment<IPointGeo> onSegment){
         Double minDistance     = Double.POSITIVE_INFINITY;
         IPointGeo closestPoint = null;
         onSegment              = null;
         //System.out.println("Topology.projectPoint:::# nodes=");
         for( Segment segment : line.getSegments() ){
            IPointGeo projected = projectPoint(point,segment);
            Double distance = Calculations.getDistance2D(point,projected);
            if ( distance < minDistance ){
               minDistance  = distance;
               closestPoint = projected;
               onSegment    = segment;
            }
         }
         return closestPoint;
     }
     public static  PointGeo projectedPoint(IPointGeo point, Double bearing, Double distance){
         Double lat = Math.asin(Math.sin(Calculations.toRadians(point.getLatitude()))*Math.cos(distance / Calculations.EARTH_RADIOUS)
                              + Math.cos(Calculations.toRadians(point.getLatitude()))*Math.sin(distance / Calculations.EARTH_RADIOUS) * 
                                Math.cos(Calculations.toRadians(bearing)));
          
         Double lon = Calculations.toRadians(point.getLongitude()) +
                      Math.atan2(Math.sin(Calculations.toRadians(bearing))*Math.sin(distance/Calculations.EARTH_RADIOUS)*Math.cos(Calculations.toRadians(point.getLatitude())),
                                 Math.cos(distance / Calculations.EARTH_RADIOUS) - Math.sin(Calculations.toRadians(point.getLatitude()))*Math.sin(lat));
         return new PointGeo(Calculations.toDegrees(lat),Calculations.toDegrees(lon));
     }
     public static boolean intersects(BBox bBox1, BBox bBox2){
         return !(bBox2.getWest()  > bBox1.getEast()  || bBox2.getEast()  < bBox1.getWest() || 
                  bBox2.getSouth() > bBox1.getNorth() || bBox2.getNorth() < bBox1.getSouth()
                 );   
     }
     public static Iterable<IPointGeo> getNodesBetweenPoints(IPointGeo from, IPointGeo to, IPolyline<IPointGeo> path){
           ArrayList<Segment<IPointGeo>> segments = path.getSegments();
           ArrayList<IPointGeo> result = new ArrayList<IPointGeo>();
           int fromIndex = -1;
           int toIndex   = -1;
           for(int i=0; i< segments.size(); i++){
              if( Calculations.getDistance2D(from, segments.get(i)) < Calculations.EPS_LENGTH){
                if (fromIndex > -1 && toIndex > -1 && toIndex <= fromIndex){}
                else fromIndex = i;
              }
              if( Calculations.getDistance2D(to,   segments.get(i)) < Calculations.EPS_LENGTH){
                if (fromIndex > -1 && toIndex > -1 && toIndex <= fromIndex){}
                else toIndex = i;              
              }
           }
           if ( fromIndex == -1 || toIndex == -1 ) return result;
           if ( fromIndex == toIndex -1 ) {
               result.add(segments.get(fromIndex).getEndPoint());}
           else if ( fromIndex -1 == toIndex ) {
               result.add(segments.get(toIndex).getEndPoint());
           }
           else if ( fromIndex < toIndex ) {
               for(int i=fromIndex;i<toIndex;i++){
                   result.add(segments.get(i).getEndPoint());
               }
           }
           else if ( toIndex < fromIndex ) {
               for(int i=fromIndex; i > toIndex;i--){
                  result.add(segments.get(i).getStartPoint());
               }
           }
           return result;
     }
}
