
package sml.geoUtils.geometry;


public class BBox {
    private Double north;   
    private Double south;
    private Double east;
    private Double west;
    private Double minElevation;
    private Double maxElevation;
    private boolean initialized;
    public BBox(){
       this.initialized = false;
    }
    public BBox(Iterable<IPointGeo> pointsToCover){
       for (IPointGeo pointToCover : pointsToCover){
           extendToCover(pointToCover);
       }
    }
    public void setNorth(Double north){this.north = north;}
    public void setSouth(Double south){this.south = south;}
    public void setWest(Double west)  {this.west  = west;}
    public void setEast(Double east)  {this.east  = east;}
    public Double getNorth(){return this.north;}
    public Double getSouth(){return this.south;}
    public Double getWest() {return this.west;}
    public Double getEast() {return this.east;}
    public final void extendToCover(IPointGeo toCover){
       if ( !this.initialized ){
          this.north = toCover.getLatitude();
          this.south = toCover.getLatitude();
          this.east  = toCover.getLongitude();
          this.west  = toCover.getLongitude();
          
          this.minElevation = toCover.getElevation();
          this.maxElevation = toCover.getElevation();
          this.initialized = true;
       }
       this.north = Math.max(this.north, toCover.getLatitude() );
       this.south = Math.min(this.south, toCover.getLatitude() );
       this.east  = Math.max(this.east,  toCover.getLongitude());
       this.west  = Math.min(this.west,  toCover.getLongitude());
       
       this.minElevation = Math.min(this.minElevation,toCover.getElevation());
       this.maxElevation = Math.max(this.maxElevation,toCover.getElevation());
    }
    public boolean isInside(IPointGeo point){
        boolean result = true;
        if ( this.north < point.getLatitude()  ){ result = false; }
        if ( this.south > point.getLatitude()  ){ result = false; }
        if ( this.east  < point.getLongitude() ){ result = false; }
        if ( this.west  > point.getLongitude() ){ result = false; }
        if ( this.maxElevation < point.getElevation() ) { result = false; }
        if ( this.minElevation > point.getElevation() ) { result = false; }
        return result; 
    }
    public boolean isInside2D(IPointGeo point){
        boolean result = true;
        if(  this.north < point.getLatitude()  ){ result = false; }
        if ( this.south > point.getLatitude()  ){ result = false; }
        if ( this.east  < point.getLongitude() ){ result = false; }
        if ( this.west  > point.getLongitude() ){ result = false; }
        return result; 
    }
    public void inflate(Double dLat, Double dLon){
         this.north = this.north + dLat;
         this.south = this.south - dLat;
         this.east  = this.east  + dLon;
         this.west  = this.west  - dLon;
    }
    public PointGeo[] getCorners(){
        PointGeo[] corners = new PointGeo[4];
        corners[0] = new PointGeo(this.getNorth(), this.getWest());
        corners[1] = new PointGeo(this.getNorth(), this.getEast());
        corners[2] = new PointGeo(this.getSouth(), this.getEast());
        corners[3] = new PointGeo(this.getSouth(), this.getWest());
        return corners;
    }
}
