
package sml.gps.datamodel;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import sml.geoUtils.geometry.IPointGeo;

public class GPSPoint implements IPointGeo {
    private Long     time;
    private Double   latitude;
    private Double   longitude;
    private Double   elevation;
    private Double   speed;
    private String   name;
    private String   description;
    private String   commenet;
    private Double   orientation;
    private Double   accuracy;
    private int      altitudeAccuracy;
    private Double   heading;    
    private String   id ="";
    private String   type;
    private String   mac;
    private String   cellId;
    private int      areaId;
    private Double   maxSpeed = -999.999;
    private int      tag = -1;
    final static String PRETEXT = "sml.gps.datamodel.GPSPoint";
    DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    public GPSPoint(String id, Double lat, Double lon){
        this.id        = id;
        this.latitude  = lat;
        this.longitude = lon;
        this.elevation =-999.999;
        this.time      = Long.MIN_VALUE;
        this.orientation      = -999.999;
        this.accuracy         = -999.999;
        this.altitudeAccuracy = -999;
        this.heading          = -999.999;        
    }    
    
    public GPSPoint(Double lat, Double lon){
        this.latitude  = lat;
        this.longitude = lon;
        this.elevation =-999.999;
        this.time      = Long.MIN_VALUE;
        this.orientation      = -999.999;
        this.accuracy         = -999.999;
        this.altitudeAccuracy = -999;
        this.heading          = -999.999;        
    }
    public GPSPoint(Double lat, Double lon, Long tim){
        super();
        this.latitude  = lat;
        this.longitude = lon;
        this.elevation = -999.999;
        this.speed     = -999.999;
        this.time      = tim;
        this.orientation      = -999.999;
        this.accuracy         = -999.999;
        this.altitudeAccuracy = -999;
        this.heading          = -999.999;

    }
    /*
    public GPSPoint(Double lat, Double lon, Long tim, Double elevation){
        this.latitude  = lat;
        this.longitude = lon;
        this.elevation = elevation;
        this.time      = tim;
    }*/
    public GPSPoint(Long time, Double lat, Double lon, Double speed){
        super();
        this.time             = time;
        this.latitude         = lat;
        this.longitude        = lon;
        this.elevation        = -999.999;
        this.speed            = speed;
        this.orientation      = -999.999;
        this.accuracy         = -999.999;
        this.altitudeAccuracy = -999;
        this.heading          = -999.999;
   }
    public GPSPoint(Long time, Double lat, Double lon, Double alt, Double speed, Double ori, Double acc, int altAcc, Double hea){
        super();     
        this.time             = time;
        this.latitude         = lat;
        this.longitude        = lon;
        this.elevation        = alt;
        this.speed            = speed;
        this.orientation      = ori;
        this.accuracy         = acc;
        this.altitudeAccuracy = altAcc;
        this.heading          = hea;
    }
    @Override
    public void   setLatitude(Double lat) {this.latitude = lat;}
    @Override
    public Double getLatitude()           {return this.latitude;}
    @Override
    public void   setLongitude(Double lon){this.longitude = lon;}
    @Override
    public Double getLongitude()          {return this.longitude;}
    @Override
    public void   setElevation(Double ele){this.elevation = ele;}
    @Override
    public Double getElevation()          {return this.elevation;}   
    public void   setTime(Long tim)       {this.time = tim;}
    public Long   getTime()               {return this.time;}  
    public String getTimeString()         {
        String out = "--";
        if ( time != Long.MIN_VALUE ){
           Date dd = new Date(this.time);
           out = df.format(dd);
        }
        return out;
    }
    public String getDescription()        {
        String out = id;
        if ( time != Long.MIN_VALUE ){
            Date dd = new Date(this.time);
            out = df.format(dd);
        }
        if ( this.tag == -1 ){
            if ( this.elevation > -900.00 ){
                out = out +":["+String.format("%.2f", this.elevation)+"] <"+this.latitude+","+this.longitude+">";
            }
            else{
                out = out + ":<"+this.latitude+","+this.longitude+">";
            }
        }
        else{
            out = out +":["+this.tag+"] <"+this.latitude+","+this.longitude+">";
        }
        if ( speed != null && speed > -900.0 ){
           out = out + " ("+String.format("%.2E", this.speed)+")";
        }
        
        return out; 
    }   
    public void   setSpeed(Double speed)  {this.speed = speed; }
    public Double getSpeed()              {return this.speed;  }
        public GPSPoint copy(){
        GPSPoint newPnt = new GPSPoint(this.getLatitude(),this.getLongitude());
        newPnt.setAccuracy(this.accuracy);
        newPnt.setElevation(this.elevation);
        newPnt.setId(this.id);
        newPnt.setMaxSpeed(this.maxSpeed);
        newPnt.setSpeed(this.speed);
        newPnt.setTime(this.time);
        newPnt.setType(this.type);
        newPnt.setMac(this.mac);
        newPnt.setCellId(this.cellId);
        newPnt.setAreaId(this.areaId);
        return newPnt;
    }
    public Double getAccuracy() {
        return accuracy;
    }

    public void setAccuracy(Double accuracy) {
        this.accuracy = accuracy;
    }
    public String getCellId() {
        return cellId;
    }

    public void setCellId(String cellId) {
        this.cellId = cellId;
    }
    public int getAreaId() {
        return areaId;
    }

    public void setAreaId(int areaId) {
        this.areaId = areaId;
    }
    public String getMac() {
        return mac;
    }

    public void setMac(String mac) {
        this.mac = mac;
    }
    public String getType()               {return this.type;}
    public void   setType(String type)    {this.type = type;}

    public Double getMaxSpeed() {
        return maxSpeed;
    }

    public void setMaxSpeed(Double maxSpeed) {
        this.maxSpeed = maxSpeed;
    }

    private void setId(String id) {
        this.id = id;
    }
    
    public void changeTag(){
        if ( false ) System.out.println(PRETEXT+".changeTag()::: initial Tag="+this.tag);
        switch (this.tag) {
            case -1:
                this.tag = 0;
                break;
            case 0:
                this.tag = 1;
                break;
            case 1:
                this.tag = -1;
                break;
            default:
                break;
        }
        if ( false ) System.out.println(PRETEXT+".changeTag()::: final Tag="+this.tag);
    }
    
    public int getTag(){
        return this.tag;
    }
}
