package sml.gps.datamodel;

import java.awt.Color;
import java.util.ArrayList;
import sml.geoUtils.Calculations;
import sml.geoUtils.geometry.Polyline;
import sml.visualizer.datamodel.Register;

public class GPSRoute extends Polyline<GPSPoint> implements Register {
    String name;
    Color  lineColor = Color.BLUE;
    int    lineWidth = 1;
    Color  symbolColor = Color.BLUE;
    int    symbolSize = 1;
    
    public GPSRoute(){}
    @Override
    public void   setName(String name) {this.name = name;}
    @Override
    public String getName()            {return this.name;}
    @Override
    public void   setLineColor(Color color){this.lineColor = color;}
    @Override
    public Color  getLineColor()           {return this.lineColor;}

    @Override
    public void setLineWidth(int width) {
        this.lineWidth = width;
    }

    @Override
    public int getLineWidth() {
        return lineWidth;
    }

    @Override
    public GPSPoint getNode(int i) {
        return this.getNodes().get(i);
    }

    @Override
    public int getType() {
        return 1;
    }
    @Override
    public GPSRoute getGPSRoute(){
        return this;
    }
    
    @Override
    public GPSPoints getGPSPoints(){
        return null;
    }    
    @Override
    public void remove(int i){
        this.getNodes().remove(i);
    }
    @Override
    public ArrayList<GPSPoint> getArrayNodes(){
        return this.getNodes();
    }       

    @Override
    public void setSymbolColor(Color color) {
        this.symbolColor = color;
    }

    @Override
    public Color getSymbolColor() {
        return this.symbolColor;
    }

    @Override
    public void setSymbolSize(int size) {
        this.symbolSize = size;
    }

    @Override
    public int getSymbolSize() {
        return this.symbolSize;
    }
    @Override
    public GPSRoute clone(){
        GPSRoute newRoute = new GPSRoute();
        for(int i=0;i<this.getNodes().size();i++){
           newRoute.getNodes().add(this.getNodes().get(i));
        }
        return newRoute;
    }
    
    public Double getDeltaTime(){
        GPSPoint ini = this.getNodes().get(0);
        GPSPoint end = this.getNodes().get(this.getNodeCount()-1);
        Double delta = (end.getTime() - ini.getTime())/(1000.0*60.0);
        return delta;
    }
    
    public void orderNodes(){
        ArrayList<GPSPoint> newNodes = new ArrayList<>();
        for(int i=0;i<this.getNodeCount();i++){
            GPSPoint pnt = this.getNodes().get(i);
            Long t = pnt.getTime();
            if ( newNodes.size() > 0 ){
                if ( t > newNodes.get(newNodes.size()-1).getTime() ){
                    newNodes.add(pnt);
                }
                else{
                    int j = 0;
                    for (GPSPoint pt : newNodes) {
                        if ( t < pt.getTime() ){ break; }
                        j = j + 1;
                    }
                    newNodes.add(j,pnt);
                }
            }
            else{
                newNodes.add(pnt);
            }
        }
        this.setNodes(newNodes);
    
    }
    public void testOrder(){
        for (int i=1;i<this.getNodeCount();i++){
            Long ots = this.getNodes().get(i-1).getTime();
            Long ts  = this.getNodes().get(i).getTime();
                if ( ts < ots  ){
                  System.out.println("PSRoute.testOrder:::: id="+this.getName()+" i="+i+"/"+this.getNodeCount()+" ts="+ts+" ots="+ots);
                }
        }
    
    }
    public Double getSpeed(){
        GPSPoint last = this.getNodes().get(0);
        Double avgSpeed = 0.0;
        for(int i=1;i<this.getNodeCount();i++){
            GPSPoint node = this.getNodes().get(i);
            Double dist = Calculations.getDistance2D(node, last)/1000.0;
            Double delta = (node.getTime() - last.getTime())/(1000.0*60.0*60);
            Double speed = dist/delta;
            node.setSpeed(speed);
            last = node;
            avgSpeed = avgSpeed + speed;
        }
        return avgSpeed/(1.0*this.getNodeCount());
        
    }
    public void setSpeed(){
        GPSPoint last = this.getNodes().get(0);
        for(int i=1;i<this.getNodeCount();i++){
            GPSPoint node = this.getNodes().get(i);
            Double dist = Calculations.getDistance2D(node, last)/1000.0;
            Double delta = (node.getTime() - last.getTime())/(1000.0*60.0*60);
            Double speed = dist/delta;
            node.setSpeed(speed);
            last = node;
        }    
    }
    public Double getIniEndLength(){
       GPSPoint ini = this.getNodes().get(0);
       GPSPoint end = this.getNodes().get(this.getNodeCount()-1);
       Double ieLength = Calculations.getDistance2D(ini, end)/1000.0;
       return ieLength;
    } 
    
    public GPSPoint getInterpolatePnt(Long time){
        if ( this.getNode(0).getTime() > time ) { return new GPSPoint(-1000.0,-1000.0,time);};
        if ( this.getNode(this.getNodeCount()-1).getTime() < time ) {return new GPSPoint(-1000.0,-1000.0,time);};
        int i;
        for(i=0;i<getNodeCount();i++){
            if ( getNode(i).getTime() > time ){
                break;
            }
        }
        GPSPoint pntIm1 = this.getNode(i-1);
        GPSPoint pntI   = this.getNode(i);
        Double   dTIm1  = (time-pntIm1.getTime())*1.0;
        Double   dTI    = (pntI.getTime()-time)*1.0;
        Double   dTIIm1 = pntI.getTime()-pntIm1.getTime()*1.0;
        Double   latIm1 = pntIm1.getLatitude();
        Double   latI   = pntI.getLatitude();
        Double   lonIm1 = pntIm1.getLongitude();
        Double   lonI   = pntI.getLongitude();
        Double   dLat   = latI - latIm1;
        Double   dLon   = lonI - lonIm1; 
        Double   lat    = ( latI * dTIm1 + latIm1 * dTI ) / dTIIm1;
        Double   lon    = ( lonI * dTIm1 + lonIm1 * dTI ) / dTIIm1;
        GPSPoint pnt = new GPSPoint(lat,lon,time);
        return pnt;
    }
}
