package sml.stops.datamodel;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import sml.geoUtils.Calculations;
import sml.gps.datamodel.GPSPoint;
import sml.utils.GeoDistance;

public class StopData {
   
   boolean ltrace = false; 
   private GPSPoint centroidNode;
   private ArrayList<GPSPoint> pnts = new ArrayList<>();
   
   private Double      radious = 0.0;
   private Long        iniTime;
   private Long        endTime;
   private Double      maxDist;
   private String      nextStopId;
   
   
   public StopData()           {
       //APos         = new ArrayList<ArrayList<Double>>();
       //ATimes       = new ArrayList<Long>();
       //centroidNode = new ArrayList<Double>();    
       pnts         = new ArrayList<>();
   }  
   public ArrayList<GPSPoint> getPnts(){
        return this.pnts;
   }
   public StopData(StopData cd){
        this.pnts = new ArrayList<>();
        for(int i=0;i<cd.getPnts().size();i++){
            GPSPoint cdNode = cd.getPnts().get(i);
            GPSPoint newNode = cdNode.copy();
            this.pnts.add(newNode);
        }
   }
   
   private void                        calcCentroid()    {

        Double centroid_lat = 0.0;
        Double centroid_lon = 0.0;
        for(int i=0;i<this.pnts.size();i++){
           centroid_lat = centroid_lat + this.pnts.get(i).getLatitude();
           centroid_lon = centroid_lon + this.pnts.get(i).getLongitude();
        }
        centroid_lat = centroid_lat / this.pnts.size();
        centroid_lon = centroid_lon / this.pnts.size();
        this.centroidNode = new GPSPoint(centroid_lat,centroid_lon);
   }
   private void                        calcRadious()     {
       this.calcCentroid();
       Double dist = -1.0;
       for (int i=0;i<this.pnts.size();i++){
            Double newDist = Calculations.getDistance2D(this.centroidNode,this.pnts.get(i));
            if ( newDist > dist ) dist = newDist;
       }
       this.radious = dist;
   }
   private void                        calcMaxDistance() {
       Double dist = -1.0;
       for (int i=0;i<this.pnts.size();i++){
           for (int j=i+1;j<this.pnts.size();j++){
               Double newDist = Calculations.getDistance2D(this.pnts.get(j),this.pnts.get(i));
               if ( newDist > dist ) dist = newDist;
           }
       }
       this.maxDist = dist;
   }
   private void                        calcIniTime()     {
       this.iniTime = Long.MAX_VALUE;
          for(int i=0;i<this.pnts.size();i++){if ( pnts.get(i).getTime() < this.iniTime) this.iniTime = pnts.get(i).getTime();
       }
       if ( this.pnts.get(0).getTime() != this.iniTime ){
          System.out.println("ClusteringData id="+this.iniTime+" not ordered in time ATimes.get(0)="+this.pnts.get(0).getTime()+", iniTime="+this.iniTime);
       }
       this.iniTime = this.pnts.get(0).getTime();
   }
   private void                        calcEndTime()     {
       this.endTime = Long.MIN_VALUE;
       for(int i=0;i<this.pnts.size();i++){if ( this.pnts.get(i).getTime() > this.endTime) this.endTime = this.pnts.get(i).getTime();}
       if ( this.endTime-this.pnts.get(this.pnts.size()-1).getTime() != 0 ){
          System.out.println("ClusteringData id="+this.iniTime+" not ordered in time ATimes.get(last)="+this.pnts.get(this.pnts.size()-1).getTime()+", endTime="+this.endTime);
       }
       this.endTime = this.pnts.get(this.pnts.size()-1).getTime();
   }     
   public void                         addNode(GPSPoint newNode)                    {
       if ( ltrace ) System.out.println("addNode::in StopData "+this.iniTime+" size="+this.pnts.size());
       boolean done = false;
       for(int i=0; i<this.pnts.size();i++){
           if ( this.pnts.get(i).getTime() == newNode.getTime() ){
               this.pnts.set(i,newNode);
               done = true;
               break;
           }
           else if ( this.pnts.get(i).getTime() >  newNode.getTime() ){
               this.pnts.add(i, newNode);

               done = true;
               break;
           }
       }
       if ( !done ){
          this.pnts.add(newNode);
       }
       calcCentroid();
       calcRadious();
       if ( ltrace ) System.out.println("addNode::in ClusterData "+this.iniTime+" size="+this.pnts.size());
   }
   public void                         addNode(GPSPoint newNode, boolean beginning) {
       if ( ltrace ) System.out.println("addNode::in ClusterData "+this.iniTime+" size="+this.pnts.size());
          if ( beginning ){
              this.pnts.add(0,newNode);
          }
          else{
              this.pnts.add(newNode);          
          }
          calcCentroid();
          calcRadious();
       if ( ltrace ) System.out.println("addNode::in ClusterData "+this.iniTime+" size="+this.pnts.size());
   }
   public void                         writeToFile()     {
        try {
            Long ini = this.getIniTime();
            Date d = new Date(ini*1000);
            Calendar cal = null;
            cal = Calendar.getInstance();
            cal.setTime(d);
            String dirName = cal.get(Calendar.YEAR)+"_"+(cal.get(Calendar.MONTH)+1)+"_"+ cal.get(Calendar.DAY_OF_MONTH);
            File theDir = new File(dirName);
            if ( ltrace )
               System.out.println("ClusterData.writeToFile::: theDir="+theDir+" ini="+getIniTime());
            if (!theDir.exists()) {
               if ( ltrace )
               System.out.println("Creating directory: " + dirName);
               boolean result = theDir.mkdir();  
               if ( result && ltrace ) {System.out.println("DIR created");}
            }
            File theFile = new File(dirName+"/"+this.getIniTime()+".csv");
            PrintWriter writer  = new PrintWriter(theFile, "UTF-8");
            writer.println(this.iniTime+","+this.nextStopId);
            for (int i=0;i<this.pnts.size();i++){
                GPSPoint node = this.pnts.get(i);
                writer.println(node.getTime()+","+node.getLatitude()+","+node.getLongitude());
            }
            writer.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(StopData.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(StopData.class.getName()).log(Level.SEVERE, null, ex);
        }
   
   }
   public String                       getId()           {return this.iniTime+"";}
   public String                       getInfo()         {
        String info = "#################\n";
        info = info + " id   ="+this.getId()+"\n";
        info = info + " size ="+this.getSize()+"\n";
        info = info + " iTime="+this.getIniTime()+"\n";
        int totalSecs = (int) (this.getEndTime()-this.getIniTime());
        int hours    = Math.round(totalSecs/3600);
        int partSecs = totalSecs - hours*3600;
        int mins     = Math.round(partSecs/60);
        int secs     = partSecs - mins*60;
        info = info + " ini Date="+new Date( getIniTime()* 1000 ).toString()+"\n";
        info = info + " ini Hour="+this.getIniTimeOfDay()+"\n";
        info = info + " end Date="+new Date( getEndTime()* 1000 ).toString()+"\n";
        info = info + " end Hour="+this.getEndTimeOfDay()+"\n";
        info = info + " DeltaT="+hours+"h"+mins+"m"+secs+"s\n";
        info = info + " Next Clus id="+this.nextStopId+"\n";
        info = info + " Centroid=<"+this.getCentroid().getLatitude()+","+this.getCentroid().getLongitude()+">\n";
        info = info + " Radious="+getRadious()+"\n";
        info = info + "#################\n";
        return info;
   }      
   public int                          getSize()         {return this.pnts.size();}
   public Long                         getIniTime()      {this.calcIniTime();return this.iniTime;}
   public Long                         getEndTime()      {this.calcEndTime();return this.endTime;}
   public Long                         getDeltaTime()    {
       Long DeltaTime = (this.getEndTime()-this.getIniTime())/1000;
       return DeltaTime;
   }
   public Date                         getIniDate()      {return new Date( this.getIniTime()* 1000 );}
   public Date                         getEndDate()      {return new Date( this.getEndTime()* 1000 );}
   public int                          getIniTimeOfDay() {
       Calendar cal = Calendar.getInstance();
       cal.setTime(getIniDate());
       int hour = cal.get(Calendar.HOUR_OF_DAY);
       int min  = cal.get(Calendar.MINUTE);
       int sec  = cal.get(Calendar.SECOND); 
       int timeOfDay = hour*60*60+min*60+sec;
       return timeOfDay;
   }
   public int                          getEndTimeOfDay() {
       Calendar cal = Calendar.getInstance();
       cal.setTime(getEndDate());
       int hour = cal.get(Calendar.HOUR_OF_DAY);
       int min  = cal.get(Calendar.MINUTE);
       int sec  = cal.get(Calendar.SECOND); 
       int timeOfDay = hour*60*60+min*60+sec;
       return timeOfDay;
   }   
   public Date                         getIniDayDate()   {
        return new Date((this.getIniTime()-getIniTimeOfDay())*1000); 
   }
   public Date                         getEndDayDate()   {
        return new Date((this.getEndTime()-getEndTimeOfDay())*1000); 
   }
   public int                          getIniWeekDay()   {
        Calendar c = Calendar.getInstance();
        c.setTime(this.getIniDate());
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);
        return dayOfWeek;
   }
   public int                          getEndWeekDay()   {
        Calendar c = Calendar.getInstance();
        c.setTime(this.getEndDate());
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);
        return dayOfWeek;
   }
   public GPSPoint                     getCentroid()     {this.calcCentroid();return this.centroidNode;}
   public Double                       getMaxDist()      {this.calcMaxDistance(); return this.maxDist;}
   public Double                       getRadious()      {this.calcRadious();return this.radious;}
   public String                       getNextStop()     {
       return this.nextStopId;
   }
   public void      setNextStop(String nextStopId)                     {
       this.nextStopId = nextStopId;
   }
   public boolean   compare(StopData otherClus, double maxDistance)    {
        boolean joinable = false;
        if ( Calculations.getDistance2D(this.getCentroid(),otherClus.getCentroid()) < maxDistance ){
            // &&   (otherClus.getIniTime()- this.getEndTime()) < 60*5 ){
           joinable = true;
        }
        return joinable;
   }
   public StopData  joinTwoStopData   (StopData otherStop)             {
        StopData newStop = new StopData();
        if ( ltrace ) System.out.println("joinTwoStopData:::stop1.size="+this.getSize()+" stop2.size="+otherStop.getSize());
        for (int i=0;i<this.getSize();i++){
            newStop.addNode(this.getPnts().get(i));
        }        
        for (int i=0;i<otherStop.getSize();i++){
            newStop.addNode(otherStop.getPnts().get(i));
        }
        if ( ltrace ) System.out.println("joinTwoStopData:::newStop.size="+newStop.getSize());
        return newStop;
   }
    public boolean   checkStopData  ( double  maxSpread, int minTimeI ) {
        boolean ok = true;
        if ( this.getSize() == 1 || this.getMaxDist() > maxSpread || this.getDeltaTime() < minTimeI ){ ok = false; }
        return ok;
    }
    public boolean   checkStopData (double maxSpread,
                                    int minTimeI,
                                    int iNumTestPoints,
                                    double randWalkPercent,
                                    boolean beginning ){
        boolean ok = true;
        if ( ltrace ) System.out.println("");
       
        int numTestPoints = Math.min(iNumTestPoints,this.pnts.size());
        if ( ltrace ) System.out.println("checkClusterData::: randWalkPercent="+randWalkPercent+", numTestPoints="+numTestPoints+", begining="+beginning);
        GPSPoint iniNode = null;
        GPSPoint endNode = null;
        if ( checkStopData(maxSpread,minTimeI) == false ){
            return false;
        }
        if ( beginning ){
            iniNode = this.pnts.get(0);
            endNode = this.pnts.get(numTestPoints-1);
        }
        else{
            iniNode = this.pnts.get(this.getSize()-numTestPoints);
            endNode = this.pnts.get(this.getSize()-1);       
        }
        double ieDist = Calculations.getDistance2D(iniNode, endNode);
        double totalDist = 0.0;
        int j = 1;       
        for (int i=1; i<numTestPoints;i++){
            if ( !beginning ) {j = this.getSize()-numTestPoints+i;}
            endNode = this.pnts.get(j);
            totalDist = totalDist + Calculations.getDistance2D(iniNode, endNode); 
            iniNode   = endNode;
            if ( false ) System.out.println("checkClusterData:::             "+i+"-> td("+j+")="+totalDist);
            j++;
        }
        if ( totalDist+0.0001 < (1.0+randWalkPercent)*ieDist && 
            numTestPoints > iNumTestPoints-1 ){ 
            ok=false; 
        }
        if ( ltrace ) System.out.println("checkClusterData::: ok="+ok+" tD="+totalDist+", ieD="+ieDist);
        if ( totalDist+0.0001 < ieDist ){
            System.out.println("\n");
            System.out.println("!!!!!ALARM!!!!!!");
            System.out.println("checkClusterData::: begining="+beginning+", ok="+ok+" tD="+totalDist+", ieD="+ieDist+
                  ", numTestPoints="+numTestPoints+", beginning="+beginning);
            if ( beginning ){
                iniNode = this.pnts.get(0);
                endNode = this.pnts.get(numTestPoints-1);
            }
            else{
                iniNode = this.pnts.get(this.getSize()-numTestPoints);
                endNode = this.pnts.get(this.getSize()-1);       
            }
          
            totalDist = 0.0;
            j = 1;
            for (int i=1; i<numTestPoints;i++){
                if ( !beginning ) {j = this.getSize()-numTestPoints+i;}
                endNode = this.pnts.get(j);
                totalDist = totalDist + Calculations.getDistance2D(iniNode, endNode);

                if ( true ) System.out.println("checkClusterData:::             "+i+"-> td("+j+")="+totalDist+
                       " <"+iniNode.getLatitude()+","+iniNode.getLongitude()+">  -- <"+endNode.getLatitude()+","+endNode.getLongitude()+">["+Calculations.getDistance2D(iniNode, endNode)+"]");
                iniNode   = endNode;
                j++;
            }
            if ( beginning ){
                for(int i=0;i<numTestPoints;i++){
                    GPSPoint node = this.pnts.get(i);
                    System.out.println("<"+node.getLatitude()+","+node.getLongitude()+">");
                }
            }else{
                for(int i=0;i<numTestPoints;i++){
                    GPSPoint node = this.pnts.get(this.pnts.size()-1-i);
                    System.out.println("<"+node.getLatitude()+","+node.getLongitude()+">");
                }          
            }
            System.out.println("!!!!!ALARM!!!!!!");
            System.out.println("\n");
        }        
        
        
        return ok;
    }
   public boolean   checkStopDataO ( double  maxSpread, 
                                     int     minTimeI,
                                     int     iNumTestPoints, 
                                     double  randWalkPercent, 
                                     boolean beginning  )              {
       boolean ok = true;
       if ( ltrace ) System.out.println("");
       
       int numTestPoints = Math.min(iNumTestPoints,this.pnts.size());
       if ( ltrace ) System.out.println("checkClusterData::: randWalkPercent="+randWalkPercent+", numTestPoints="+numTestPoints+", begining="+beginning);
       GPSPoint iniNode = null;
       GPSPoint endNode = null;
       if ( checkStopData(maxSpread,minTimeI) == false ){
          return false;
       }
       if ( beginning ){
            iniNode = this.pnts.get(0);
            endNode = this.pnts.get(numTestPoints-1);
       }
       else{
            iniNode = this.pnts.get(this.getSize()-numTestPoints);
            endNode = this.pnts.get(this.getSize()-1);       
       }
       double ieDist = Calculations.getDistance2D(iniNode, endNode);
       double totalDist = 0.0;
       int j = 1;       
       for (int i=1; i<numTestPoints;i++){
            if ( !beginning ) {j = this.getSize()-numTestPoints+i;}
            endNode = this.pnts.get(j);
            totalDist = totalDist + Calculations.getDistance2D(iniNode, endNode);;
            iniNode   = endNode;
            if ( false ) System.out.println("checkClusterData:::             "+i+"-> td("+j+")="+totalDist);
            j++;
       }
       if ( totalDist+0.0001 < (1.0+randWalkPercent)*ieDist && 
            numTestPoints > iNumTestPoints-1 ){ 
            ok=false; 
       }
       if ( ltrace ) System.out.println("checkClusterData::: ok="+ok+" tD="+totalDist+", ieD="+ieDist);
       if ( totalDist+0.0001 < ieDist ){
            System.out.println("\n");
            System.out.println("!!!!!ALARM!!!!!!");
            System.out.println("checkClusterData::: begining="+beginning+", ok="+ok+" tD="+totalDist+", ieD="+ieDist+
                  ", numTestPoints="+numTestPoints+", beginning="+beginning);
            if ( beginning ){
                iniNode = this.pnts.get(0);
                endNode = this.pnts.get(numTestPoints-1);
            }
            else{
                iniNode = this.pnts.get(this.getSize()-numTestPoints);
                endNode = this.pnts.get(this.getSize()-1);       
            }
          
          totalDist = 0.0;
          j = 1;
          for (int i=1; i<numTestPoints;i++){
               if ( !beginning ) {j = this.getSize()-numTestPoints+i;}
               endNode = this.pnts.get(j);
               totalDist = totalDist + Calculations.getDistance2D(iniNode, endNode);

               if ( true ) System.out.println("checkClusterData:::             "+i+"-> td("+j+")="+totalDist+
                       " <"+iniNode.getLatitude()+","+iniNode.getLongitude()+">  -- <"+endNode.getLatitude()+","+endNode.getLongitude()+">["+Calculations.getDistance2D(iniNode, endNode)+"]");
               iniNode   = endNode;
               j++;
          }
          if ( beginning ){
            for(int i=0;i<numTestPoints;i++){
                GPSPoint node = this.pnts.get(i);
                System.out.println("<"+node.getLatitude()+","+node.getLongitude()+">");
            }
          }else{
            for(int i=0;i<numTestPoints;i++){
                GPSPoint node = this.pnts.get(this.pnts.size()-1-i);
                System.out.println("<"+node.getLatitude()+","+node.getLongitude()+">");
            }          
          }
          System.out.println("!!!!!ALARM!!!!!!");
          System.out.println("\n");
       }
       return ok;
   }
}
