package sml.utils;

import sml.geoUtils.geometry.IPointGeo;
import sml.geoUtils.geometry.IPolyline;
import sml.geoUtils.geometry.PointGeo;
import sml.geoUtils.geometry.Segment;
import sml.gps.datamodel.GPSPoint;





public class Calculations {
    public static Double EARTH_RADIOUS = 6371010.0;
    public static Double EPS_LENGTH    = 0.01;
    //static GreatCircleDistanceCalculator distanceCalculator;

    public static Double getBearing(IPointGeo pt1, IPointGeo pt2){
        Double dLon = Calculations.toRadians(pt1.getLongitude()-pt2.getLongitude());
        Double dLat = Calculations.toRadians(pt1.getLatitude()-pt2.getLatitude());
        
        Double y = Math.sin(dLon) * Math.cos(Calculations.toRadians(pt2.getLatitude()));
        Double x = Math.cos(Calculations.toRadians(pt1.getLatitude())) * Math.sin(Calculations.toRadians(pt2.getLatitude()))
                - Math.sin(Calculations.toRadians(pt1.getLatitude())) * Math.cos(Calculations.toRadians(pt2.getLatitude())) * Math.cos(dLon);
        return Calculations.toDegrees(Math.atan2(y, x));
    }
    public static Double getBearing(Segment<IPointGeo> segment){
        return Calculations.getBearing(segment.getStartPoint(), segment.getEndPoint());
    };
    public static Double getDistance2D(GPSPoint pt1, GPSPoint pt2){
        return GeoDistance.CalcDistance(pt1,pt2);
    }
    public static Double getDistance2D(IPointGeo pt1, IPointGeo pt2){
        //System.out.println("pt1=<"+pt1.getLatitude()+","+pt1.getLongitude()+">");
        //System.out.println("pt1=<"+pt2.getLatitude()+","+pt2.getLongitude()+">");
        if( pt1 == null ){ System.err.println("Calculations.getDistance:::pt1=null"); System.exit(1);}
        if( pt2 == null ){ System.err.println("Calculations.getDistance:::pt2=null"); System.exit(1);}
        return Calculations.getDistance2D(pt1, pt2);
    }
    public static Double getPathLength(IPointGeo from, IPointGeo to, Segment<IPointGeo> path){
        return Calculations.getDistance2D(from, to);
    }
    public static Long getPathTime(IPointGeo from, IPointGeo to, Segment<IPointGeo> path){
        Double length = getPathLength(from, to, path);
        return 10L;
    }
    public static Double getLength(IPolyline<IPointGeo> line){
        Double length = 0.0;
        for(Segment segment : line.getSegments()){
           length = length + segment.getLength();
        }
        return length;
    }
    public static PointGeo Averaging(Iterable<IPointGeo> points){
       int count = 0;
       Double lat = 0.0;
       Double lon = 0.0;
       for(IPointGeo point : points){
           lat = lat + point.getLatitude();
           lon = lon + point.getLongitude();
           count = count + 1;
       }
       return new PointGeo(lat/count, lon/count, -1.0092);
    }
    public static Double toRadians(Double angle){
       return angle * Math.PI / 180.0;
    }
    public static double toDegrees(Double angle){
       return angle * 180.0 / Math.PI;
    }
}
