package sml.visualizer.gui.functions;

import java.util.ArrayList;
import sml.gps.datamodel.GPSPoint;
import sml.gps.datamodel.GPSRoute;
import sml.stops.datamodel.StopData;
import sml.utils.GeoDistance;

public class FindStops {
   
   private ArrayList<ArrayList<Double>>  APos          = new ArrayList<ArrayList<Double>>();
   private ArrayList<Long>               ATimes        = new ArrayList<Long>();
   private ArrayList<Double>             AVels         = new ArrayList<Double>();
   private ArrayList<StopData>           stops         = new ArrayList<StopData>();
   private ArrayList<ArrayList<Integer>> AStopIndxs    = new ArrayList<ArrayList<Integer>>();
   // For the loop...
   private StopData           currentStop = null;
   private ArrayList<Integer> stopIndxs   = new ArrayList<Integer>();
   ////////////////////////////////////////////////////////////////////
   private double maxSpread       = 0.4;
   private double minTime         = 0.4;
   private int    minTimeI        = 24;
   private double maxVel          = 2.0;
   private int    numTestPoints   = 4;
   private double randWalkPercent = 0.35;
   private int    oldI = -1;
   public FindStops()                                  {
         APos    = new ArrayList<>();
         ATimes  = new ArrayList<>();
         AVels   = new ArrayList<>();
   };
   public FindStops(double maxSpread,       // maximum Spreading for clusters
                    double minTime,         // minimum time for a LoI
                    double maxVel,          // maximum velocity for initial cluster identification
                    int numTestPoints,      // # test points for linearity
                    double randWalkPercent  // measure of the random walk-enes.
         )                                              {
         APos    = new ArrayList<>();
         ATimes  = new ArrayList<>();
         AVels   = new ArrayList<>();
         this.maxSpread       = maxSpread;
         this.minTime         = minTime;
         this.minTimeI        = (int) Math.round(minTime*60);
         this.maxVel          = maxVel;
         this.numTestPoints   = numTestPoints;
         this.randWalkPercent = randWalkPercent; 
   }
   
   public  void addRawData(GPSRoute rd)                  {
        ArrayList<ArrayList<Double>> newAPos   = new ArrayList<>();
        ArrayList<Long>              newATimes = new ArrayList<>();
        ArrayList<Double>            newAVels  = new ArrayList<>();
        for(int i=0;i<rd.getNodeCount();i++){
            ArrayList<Double> pos = new ArrayList<>();
            pos.add(rd.getNodes().get(i).getLatitude());
            pos.add(rd.getNodes().get(i).getLongitude());  
            newAPos.add(pos);
            newATimes.add(rd.getNodes().get(i).getTime()/1000);
            newAVels.add(rd.getNodes().get(i).getSpeed());
        }
        for (int j=0;j<newATimes.size();j++){
            Long              newTime = newATimes.get(j);
            ArrayList<Double> newNode = newAPos.get(j);
            Double            newVel  = newAVels.get(j);
            boolean done = false;
            if ( oldI >= 0 ){ if ( this.ATimes.get(oldI) > newTime ){ oldI = 0;} }
            else                                                    { oldI = 0;}
            for(int i=oldI; i<this.ATimes.size();i++){
               if ( this.ATimes.get(i) == newTime ){
                   this.APos.set(i,  newNode);
                   this.AVels.set(i, newVel);
                   done = true;
                   oldI = i;
                   break;
               }
               else if ( this.ATimes.get(i) >  newTime ){
                   this.ATimes.add(i, newTime);
                   this.APos.add(i,newNode);
                   this.AVels.add(i,newVel);
                   done = true;
                   oldI = i;
                   break;
               }
            } 
            if ( !done ){
              oldI = this.ATimes.size();
              this.ATimes.add(newTime);
              this.APos.add(newNode);
              this.AVels.add(newVel);
            }
        }
        /*
        System.out.println("FindStops.addRawData:::size(output)="+ATimes.size());  
        for(int i=0;i<this.ATimes.size();i++){
           System.out.println("time="+this.ATimes.get(i)+" <"+this.APos.get(i).get(0)+","+this.APos.get(i).get(1)+"> ("+this.AVels.get(i)+")");
        }*/
   }
   private void calcZeroVelClusters()                   {
       double vel;
       boolean ltrace = false;
       Long time;
       Long lastTime = -1L;
       Long minStop  = Math.round(maxSpread/maxVel*3600L); 
       //Long minStop = 5*60L;
       System.out.println("minStop="+minStop);
       boolean inStop = false;
       ArrayList<Double>  node     = new ArrayList<>();
       ArrayList<Double>  lastNode = new ArrayList<>();
       ArrayList<Double>  nextNode = new ArrayList<>();
       currentStop  = null; 
       for (int i=0;i<ATimes.size();i++){
            time = ATimes.get(i);
            vel  = AVels.get(i);
            node = APos.get(i);
            //if ( time == 1378874028 || lastTime == 1378874028) ltrace = true;
            //else ltrace = false;
            if (ltrace)
              System.out.println("i:: "+i+" time="+time+", vel="+vel+", inStop"+inStop+", currentCluster="+currentStop);
            if ( vel < maxVel ){
                if (ltrace) System.out.println("vel<maxVel");
                inStop = addRawPointToCluster(time, i, node, lastNode, inStop);
            }
            else if ( time-lastTime > minStop && !inStop ){
                if (ltrace) System.out.println("in the past!!");
                inStop = addRawPointToCluster(time, i, node, lastNode, inStop);
            }
            else if ( i< ATimes.size()-1 && ATimes.get(i+1)-ATimes.get(i) > minStop ){
                if (ltrace) System.out.println("in the future!!");
                nextNode  = APos.get(i+1);
                inStop = addRawPointToCluster(time, i, node, nextNode, inStop);
            }
            else{
                inStop = false;
            }
            lastNode = node;
            lastTime = time;
       }
       for(int i=0;i<stops.size();i++){
           if (stops.get(i).getRadious()> maxSpread )
              System.out.println(i+": radious"+stops.get(i).getRadious()+"!!!!!!!!!!!!!!!!!!!!");
       }
   }
   private boolean addRawPointToCluster(long time, int i,
                         ArrayList<Double> node, ArrayList<Double> otherNode, 
                         boolean inStop)             {
             boolean lInStop = inStop;
             boolean ltrace = false;
             GPSPoint pnt = new GPSPoint(node.get(0),node.get(1),time*1000);
             if ( i == 860 || i == 859 ) ltrace = true;
             else ltrace = false;
             if ( ltrace ){
                System.out.println("time="+time+" i="+i+" inStop="+inStop+" otherNode.size()="+otherNode.size()+
                        " node=<"+node.get(0)+","+node.get(1)+"> otherNode="+otherNode.get(0)+","+otherNode.get(1)+"> D="+GeoDistance.calcDist(node,otherNode));
             }
             if ( otherNode.size() > 0 ){ 
                 if ( GeoDistance.calcDist(node,otherNode) < maxSpread ){
                     
                     if ( !lInStop ){
                         lInStop = true;
                         currentStop = new StopData();
                         stopIndxs   = new ArrayList<>();
                         AStopIndxs.add(stopIndxs);
                         stops.add(currentStop);
                     }
                     
                     currentStop.addNode(pnt);
                     stopIndxs.add(i);
                 }
                 else{
                     lInStop = false;
                 }
             }
             else{
                 if ( !lInStop ){
                     lInStop = true;
                     currentStop = new StopData();
                     stopIndxs   = new ArrayList<Integer>();
                     AStopIndxs.add(stopIndxs);
                     stops.add(currentStop);
                 }
                 currentStop.addNode(pnt);
                 stopIndxs.add(i);             
             }
             if ( lInStop && ltrace ) System.out.println("point added to cluster cluster.size()="+currentStop.getSize());
             return lInStop;
   }
   private void joinStops()                          {
       int oldStopsSize     = this.stops.size();
       int newStopsSize     = -this.stops.size();
       System.out.println("joinClusters::: (on input) clusterSize="+oldStopsSize);
       int pass = 0;
       while ( newStopsSize != oldStopsSize ){
           System.out.println("       joinStops::: pass="+pass+" size="+newStopsSize);
           int initialStop = 0;
           this.stops   = this.joinStopsSinglePass(this.stops,initialStop);
           initialStop = 1;
           this.stops   = this.joinStopsSinglePass(this.stops,initialStop);
           oldStopsSize = newStopsSize;
           newStopsSize = this.stops.size();
           pass++;
       }
       System.out.println("joinStops::: (on output) stopsSize="+newStopsSize);
   }
   private ArrayList<StopData> joinStopsSinglePass(ArrayList<StopData> clusters, int initialStop){
       int limit = clusters.size();
       ArrayList<StopData> newStops = new ArrayList<StopData>();
       //for (int k=0;k<1;k++){
           for (int c1=initialStop;c1<limit-1;c1=c1+2){
               StopData stop1 = stops.get(c1);
               StopData stop2 = stops.get(c1+1);
               if ( c1 == 1 ){
                 newStops.add(stops.get(0));
               }
               boolean joined = false;
               StopData stop = null;
               if ( stop1.compare(stop2,maxSpread) ) {
                   stop = stop1.joinTwoStopData(stop2);
                   joined = true;
                   if ( stop.checkStopData(maxSpread,minTimeI) ){ joined = true; }
               }
               if ( joined ){
                   newStops.add(stop);
               }
               else{
                   newStops.add(stop1);
                   newStops.add(stop2);
               }
               if ( c1 == limit-3 ){
                   newStops.add(stops.get(limit-1));
               }

           //}
       }
       return newStops;
   }    
   private void expandStops()                        {
        int changes = 0;
        for(int c=0;c<this.stops.size();c++){
           //System.out.println("expandClusters::: cluster ="+c);
           changes = this.expandSingleStop(c);
        }
   }
   private int  expandSingleStop(int stopIndx)       {
       boolean ltrace = false;
       int oldSize = stops.get(stopIndx).getSize();
       int newSize = -1;
       int changes = -1;
       int k=0;
       while ( newSize != oldSize ){
           StopData stop = stops.get(stopIndx);
           int iniIndx = AStopIndxs.get(stopIndx).get(0);
           int endIndx = AStopIndxs.get(stopIndx).get(AStopIndxs.get(stopIndx).size()-1);
           StopData newTempStop = new StopData(stop);
           if ( ltrace ){
              System.out.print("change "+k+":        [");
              for(int j=0;j<AStopIndxs.get(stopIndx).size();j++){
                  System.out.print(AStopIndxs.get(stopIndx).get(j)+",");
              }
              System.out.print("\b\b] size="+stop.getSize()+"  \n");
           }
           if ( iniIndx > 0 ) {
               newTempStop = expandSingleClusterBegining(stop, ATimes.get(iniIndx-1),APos.get(iniIndx-1));
               if ( newTempStop.checkStopData(maxSpread, minTimeI, numTestPoints, randWalkPercent, true) ){
                  if ( ltrace ) System.out.println("B");
                  stops.set(stopIndx, newTempStop);
                  AStopIndxs.get(stopIndx).add(0, iniIndx-1);
               }
           }
           stop = stops.get(stopIndx);
           if ( endIndx < ATimes.size()-1 ) {
               newTempStop = expandSingleClusterEnd(stop, ATimes.get(endIndx+1),APos.get(endIndx+1));
               if ( newTempStop.checkStopData(maxSpread, minTimeI, numTestPoints, randWalkPercent, false) ){
                  if ( ltrace ) System.out.println("E");
                  stops.set(stopIndx, newTempStop);
                  AStopIndxs.get(stopIndx).add(endIndx+1);
               }
           }
           if ( ltrace ) System.out.println("");
           oldSize = newSize;
           newSize =  stops.get(stopIndx).getSize();
           changes++;
           k++;
           if ( k > 1000000000 ) break;
       }
       return changes;
   }
   private StopData expandSingleClusterBegining(StopData stop, Long time, ArrayList<Double> pos)     {
        StopData newTempStop = new StopData(stop);
        GPSPoint pnt = new GPSPoint(pos.get(0),pos.get(1),time*1000);
        newTempStop.addNode(pnt, true);
        return newTempStop;
   }
   private StopData expandSingleClusterEnd     (StopData stop, Long time, ArrayList<Double> pos)     {
        StopData newTempClus = new StopData(stop);
        GPSPoint pnt = new GPSPoint(pos.get(0),pos.get(1),time*1000);
        newTempClus.addNode(pnt, false);
        return newTempClus;
   }  
   private void removeShortTimeStops(double minTime) {
       ArrayList<Integer> indxsToErase = new ArrayList<Integer>();
       for(int c=0;c<this.stops.size();c++){
          if ( this.stops.get(c).getDeltaTime() < minTime*60 && c!=0 && c!=(this.stops.size()-1) ){
             indxsToErase.add(c);
            // System.out.println("removeShortTimeClusters:: this.clusters.getDeltaTime("+c+")="+this.clusters.get(c).getDeltaTime());
          }
       }
       for(int c=stops.size()-1;c>=0;c--){
           if ( indxsToErase.contains(c) ) this.stops.remove(c);
       }
   }
   private void writeAllStops()                      {
       int wclus = 0;
       for (int c=0;c<this.stops.size();c++){          
           StopData cluster = this.stops.get(c);
           //if ( cluster.checkClusteringData() ){
           cluster.writeToFile();
           wclus++;
           //}
       }   
   } 
   private ArrayList<StopData> orderStops()       {
       ArrayList<StopData> orderedStops = new ArrayList<StopData>();
       for(int c1=0;c1<this.stops.size();c1++){
          StopData stop = stops.get(c1);
          Long stopIni = stop.getIniTime();
          boolean alreadyAdded = false;
          for (int c2=0;c2<orderedStops.size();c2++){
              StopData clusO = orderedStops.get(c2);
              Long clusOIni = clusO.getIniTime();
              if ( clusOIni>stopIni ){
                 orderedStops.add(c2,stop);
                 alreadyAdded = true;
              }
          }
          if (!alreadyAdded){
             orderedStops.add(stop);
          }
       }
       return orderedStops;
   }
   private void findAllNextStops()                   {
        ArrayList<StopData> orderedStops = orderStops();
        for(int c=0;c<orderedStops.size()-1;c++){
           StopData clus   = orderedStops.get(c);
           StopData clusp1 = orderedStops.get(c+1);
           if ( clus.getEndTime() < clusp1.getIniTime() ){
              clus.setNextStop(clusp1.getId());
           }
        }
   }
   public  ArrayList<StopData> doStops()                          {
       stops = new ArrayList<StopData>();
       this.calcZeroVelClusters();
       System.out.println("Zero Vel Stops identified #(Clus)="+this.stops.size());
       this.expandStops();
       System.out.println("Stops expanded #(Clus)="+this.stops.size());
       this.joinStops();
       System.out.println("Stops joined #(Clus)="+this.stops.size());
       this.removeShortTimeStops(this.minTime);
       System.out.println("Small Stops (<"+minTime+" mins) erased #(Clus)="+this.stops.size());
       this.findAllNextStops();
       return stops;
   }
}
